package kr.co.vividnext.sodalive.admin.content.curation.tag

import kr.co.vividnext.sodalive.admin.content.curation.AddItemToCurationRequest
import kr.co.vividnext.sodalive.admin.content.curation.RemoveItemInCurationRequest
import kr.co.vividnext.sodalive.admin.content.curation.UpdateCurationItemOrdersRequest
import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/admin/audio-content/tag/curation")
@PreAuthorize("hasRole('ADMIN')")
class AdminHashTagCurationController(private val service: AdminHashTagCurationService) {
    @GetMapping
    fun getContentHashTagCurationList() = ApiResponse.ok(service.getContentHashTagCurationList())

    @PostMapping
    fun createContentHashTagCuration(
        @RequestBody request: CreateContentHashTagCurationRequest
    ) = ApiResponse.ok(service.createContentHashTagCuration(request))

    @PutMapping
    fun updateContentHashTagCuration(
        @RequestBody request: UpdateContentHashTagCurationRequest
    ) = ApiResponse.ok(service.updateContentHashTagCuration(request))

    @PutMapping("/orders")
    fun updateContentHashTagCurationOrders(
        @RequestBody request: UpdateContentHashTagCurationOrderRequest
    ) = ApiResponse.ok(service.updateContentHashTagCurationOrders(request.ids), "수정되었습니다.")

    @GetMapping("/items")
    fun getHashTagCurationItemList(
        @RequestParam curationId: Long
    ) = ApiResponse.ok(service.getHashTagCurationItemList(curationId = curationId))

    @GetMapping("/search/content")
    fun searchHashTagCurationContentItem(
        @RequestParam curationId: Long,
        @RequestParam searchWord: String
    ) = ApiResponse.ok(service.searchHashTagCurationContentItem(curationId, searchWord))

    @PostMapping("/add/item")
    fun addItemToHashTagCuration(
        @RequestBody request: AddItemToCurationRequest
    ) = ApiResponse.ok(service.addItemToHashTagCuration(request), "큐레이션 아이템을 등록했습니다.")

    @PutMapping("/remove/item")
    fun removeItemInHashTagCuration(
        @RequestBody request: RemoveItemInCurationRequest
    ) = ApiResponse.ok(service.removeItemInHashTagCuration(request), "큐레이션 아이템을 제거했습니다.")

    @PutMapping("/orders/item")
    fun updateItemInHashTagCurationOrders(
        @RequestBody request: UpdateCurationItemOrdersRequest
    ) = ApiResponse.ok(service.updateItemInHashTagCurationOrders(request), "수정되었습니다.")
}
