package kr.co.vividnext.sodalive.admin.content.curation.tag

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.content.curation.QSearchCurationItemResponse
import kr.co.vividnext.sodalive.admin.content.curation.SearchCurationItemResponse
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.main.curation.tag.ContentHashTagCuration
import kr.co.vividnext.sodalive.content.main.curation.tag.QContentHashTagCuration.contentHashTagCuration
import kr.co.vividnext.sodalive.content.main.curation.tag.QContentHashTagCurationItem.contentHashTagCurationItem
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

interface AdminHashTagCurationRepository :
    JpaRepository<ContentHashTagCuration, Long>,
    AdminHashTagCurationQueryRepository

interface AdminHashTagCurationQueryRepository {
    fun getContentHashTagCurationList(): List<GetAdminContentHashTagCurationResponse>
    fun searchHashTagCurationContentItem(curationId: Long, searchWord: String): List<SearchCurationItemResponse>
    fun isExistsTag(tag: String): Boolean
}

@Repository
class AdminHashTagCurationQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) : AdminHashTagCurationQueryRepository {
    override fun getContentHashTagCurationList(): List<GetAdminContentHashTagCurationResponse> {
        return queryFactory
            .select(
                QGetAdminContentHashTagCurationResponse(
                    contentHashTagCuration.id,
                    contentHashTagCuration.tag,
                    contentHashTagCuration.isAdult
                )
            )
            .from(contentHashTagCuration)
            .where(contentHashTagCuration.isActive.isTrue)
            .orderBy(contentHashTagCuration.orders.asc())
            .fetch()
    }

    override fun searchHashTagCurationContentItem(
        curationId: Long,
        searchWord: String
    ): List<SearchCurationItemResponse> {
        return queryFactory
            .select(
                QSearchCurationItemResponse(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("/").prepend(imageHost)
                )
            )
            .from(audioContent)
            .leftJoin(contentHashTagCurationItem)
            .on(
                audioContent.id.eq(contentHashTagCurationItem.content.id)
                    .and(contentHashTagCurationItem.curation.id.eq(curationId))
            )
            .where(
                audioContent.duration.isNotNull
                    .and(audioContent.member.isNotNull)
                    .and(audioContent.isActive.isTrue)
                    .and(audioContent.title.contains(searchWord))
                    .and(
                        contentHashTagCurationItem.id.isNull
                            .or(contentHashTagCurationItem.isActive.isFalse)
                    )
            )
            .fetch()
    }

    override fun isExistsTag(tag: String): Boolean {
        return queryFactory
            .select(contentHashTagCuration.id)
            .from(contentHashTagCuration)
            .where(
                contentHashTagCuration.tag.eq(tag),
                contentHashTagCuration.isActive.isTrue
            )
            .fetch().isNotEmpty()
    }
}
