package kr.co.vividnext.sodalive.admin.content.series.recommend

import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RequestPart
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@PreAuthorize("hasRole('ADMIN')")
@RequestMapping("/admin/audio-content/series/recommend")
class AdminRecommendSeriesController(private val service: AdminRecommendSeriesService) {
    @GetMapping
    fun getRecommendSeriesList(@RequestParam isFree: Boolean) = ApiResponse.ok(
        service.getRecommendSeriesList(isFree = isFree)
    )

    @PostMapping
    fun createRecommendSeries(
        @RequestPart("image") image: MultipartFile,
        @RequestPart("request") requestString: String
    ) = ApiResponse.ok(service.createRecommendSeries(image, requestString))

    @PutMapping
    fun modifyRecommendSeries(
        @RequestPart("image", required = false) image: MultipartFile? = null,
        @RequestPart("request") requestString: String
    ) = ApiResponse.ok(service.updateRecommendSeries(image, requestString))

    @PutMapping("/orders")
    fun updateRecommendSeriesOrders(
        @RequestBody request: UpdateRecommendSeriesOrdersRequest
    ) = ApiResponse.ok(service.updateRecommendSeriesOrders(request.ids))
}
