package kr.co.vividnext.sodalive.admin.live

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.live.recommend.QRecommendLiveCreatorBanner.recommendLiveCreatorBanner
import kr.co.vividnext.sodalive.live.recommend.RecommendLiveCreatorBanner
import kr.co.vividnext.sodalive.live.room.LiveRoom
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.domain.Pageable
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
class AdminLiveRoomQueryRepository(private val queryFactory: JPAQueryFactory) {
    fun getLiveRoomList(imageHost: String): List<GetLiveResponseItem> {
        return queryFactory
            .select(
                QGetLiveResponseItem(
                    liveRoom.id,
                    liveRoom.title,
                    liveRoom.notice,
                    Expressions.constant(0),
                    liveRoom.member.nickname,
                    liveRoom.coverImage.coalesce("profile/default-profile.png")
                        .prepend("/")
                        .prepend(imageHost),
                    liveRoom.channelName.coalesce(""),
                    liveRoom.type,
                    liveRoom.password,
                    liveRoom.isAdult
                )
            )
            .from(liveRoom)
            .innerJoin(liveRoom.member, member)
            .where(liveRoom.isActive.isTrue)
            .orderBy(liveRoom.channelName.desc(), liveRoom.beginDateTime.asc())
            .fetch()
    }

    fun getLiveRoomTotalCount(): Int {
        return queryFactory
            .select(liveRoom.id)
            .from(liveRoom)
            .where(liveRoom.isActive.isTrue)
            .fetch()
            .size
    }

    fun getRecommendCreatorTotalCount(): Int {
        return queryFactory
            .select(recommendLiveCreatorBanner.id)
            .from(recommendLiveCreatorBanner)
            .fetch()
            .size
    }

    fun getRecommendCreatorList(pageable: Pageable): List<RecommendLiveCreatorBanner> {
        return queryFactory
            .selectFrom(recommendLiveCreatorBanner)
            .offset(pageable.offset)
            .limit(pageable.pageSize.toLong())
            .orderBy(recommendLiveCreatorBanner.orders.asc(), recommendLiveCreatorBanner.id.desc())
            .fetch()
    }

    fun getNotStartedLive4hoursPassed(): List<LiveRoom> {
        return queryFactory
            .selectFrom(liveRoom)
            .where(
                liveRoom.isActive.isTrue
                    .and(liveRoom.channelName.isNull)
                    .and(liveRoom.beginDateTime.loe(LocalDateTime.now().minusHours(4)))
            )
            .fetch()
    }
}
