package kr.co.vividnext.sodalive.can.charge

import kr.co.vividnext.sodalive.can.payment.PaymentGateway
import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.marketing.AdTrackingHistoryType
import kr.co.vividnext.sodalive.marketing.AdTrackingService
import kr.co.vividnext.sodalive.member.Member
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import java.time.LocalDateTime

@RestController
@RequestMapping("/charge")
class ChargeController(
    private val service: ChargeService,
    private val trackingService: AdTrackingService
) {

    @PostMapping
    fun charge(
        @RequestBody chargeRequest: ChargeRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        ApiResponse.ok(service.charge(member, chargeRequest))
    }

    @PostMapping("/verify")
    fun verify(
        @RequestBody verifyRequest: VerifyRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        val response = service.verify(memberId = member.id!!, verifyRequest)
        trackingCharge(member, response)
        ApiResponse.ok(Unit)
    }

    @PostMapping("/verify/hecto")
    fun verifyHecto(
        @RequestBody verifyRequest: VerifyRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        val response = service.verifyHecto(memberId = member.id!!, verifyRequest)
        trackingCharge(member, response)
        ApiResponse.ok(Unit)
    }

    @PostMapping("/apple")
    fun appleCharge(
        @RequestBody chargeRequest: AppleChargeRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        ApiResponse.ok(service.appleCharge(member, chargeRequest))
    }

    @PostMapping("/apple/verify")
    fun appleVerify(
        @RequestBody verifyRequest: AppleVerifyRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        val response = service.appleVerify(memberId = member.id!!, verifyRequest)
        trackingCharge(member, response)
        ApiResponse.ok(Unit)
    }

    @PostMapping("/google")
    fun googleCharge(
        @RequestBody request: GoogleChargeRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        if (request.paymentGateway == PaymentGateway.GOOGLE_IAP) {
            val chargeId = service.googleCharge(
                member = member,
                title = request.title,
                chargeCan = request.chargeCan,
                price = request.price,
                currencyCode = request.currencyCode,
                productId = request.productId,
                purchaseToken = request.purchaseToken,
                paymentGateway = request.paymentGateway
            )

            val response = service.processGoogleIap(
                memberId = member.id!!,
                chargeId = chargeId,
                productId = request.productId,
                purchaseToken = request.purchaseToken,
                paymentGateway = request.paymentGateway
            )

            trackingCharge(member, response)
            ApiResponse.ok(Unit)
        } else {
            throw SodaException("결제정보에 오류가 있습니다.")
        }
    }

    private fun trackingCharge(
        member: Member,
        response: ChargeCompleteResponse
    ) {
        if (
            !member.activePid.isNullOrBlank() &&
            member.partnerExpirationDatetime?.isAfter(LocalDateTime.now()) == true
        ) {
            trackingService.saveTrackingHistory(
                pid = member.activePid!!,
                type = if (response.isFirstCharged) {
                    AdTrackingHistoryType.FIRST_PAYMENT
                } else {
                    AdTrackingHistoryType.REPEAT_PAYMENT
                },
                memberId = member.id!!,
                price = response.price,
                locale = response.currencyCode
            )
        }
    }
}
