package kr.co.vividnext.sodalive.can.coupon

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.core.io.InputStreamResource
import org.springframework.data.domain.Pageable
import org.springframework.http.HttpHeaders
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController
import java.net.URLEncoder
import java.nio.charset.StandardCharsets

@RestController
@RequestMapping("/can/coupon")
class CanCouponController(private val service: CanCouponService) {
    @PostMapping
    @PreAuthorize("hasRole('ADMIN')")
    fun generateCoupon(
        @RequestBody request: GenerateCanCouponRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.generateCoupon(request))
    }

    @PutMapping
    @PreAuthorize("hasRole('ADMIN')")
    fun modifyCoupon(
        @RequestBody request: ModifyCanCouponRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.modifyCoupon(request))
    }

    @GetMapping
    @PreAuthorize("hasRole('ADMIN')")
    fun getCouponList(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getCouponList(offset = pageable.offset, limit = pageable.pageSize.toLong()))
    }

    @GetMapping("/number-list")
    @PreAuthorize("hasRole('ADMIN')")
    fun getCouponNumberList(
        @RequestParam couponId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getCouponNumberList(
                couponId = couponId,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )
    }

    @GetMapping("/number-list/download")
    @PreAuthorize("hasRole('ADMIN')")
    fun downloadCouponNumberList(
        @RequestParam couponId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        val fileName = "쿠폰번호리스트.xlsx"
        val encodedFileName = URLEncoder.encode(
            fileName,
            StandardCharsets.UTF_8.toString()
        ).replace("+", "%20")
        val contentDisposition = "attachment; filename*=UTF-8''$encodedFileName"
        val headers = HttpHeaders().apply {
            add(HttpHeaders.CONTENT_DISPOSITION, contentDisposition)
        }

        val response = service.downloadCouponNumberList(couponId)
        ResponseEntity.ok()
            .headers(headers)
            .contentType(
                MediaType
                    .parseMediaType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
            )
            .body(InputStreamResource(response))
    }

    @PostMapping("/use")
    fun useCanCoupon(
        @RequestBody request: UseCanCouponRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.useCanCoupon(
                couponNumber = request.couponNumber,
                memberId = member.id!!
            )
        )
    }
}
