package kr.co.vividnext.sodalive.content.playlist

import com.querydsl.core.annotations.QueryProjection
import org.springframework.data.annotation.Id
import org.springframework.data.redis.core.RedisHash
import org.springframework.data.redis.core.index.Indexed
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter

@RedisHash("AudioContentPlaylist")
data class AudioContentPlaylist(
    @Id
    val id: Long,
    @Indexed
    val memberId: Long,
    var title: String,
    var desc: String? = null,
    var contentIdAndOrderList: List<PlaylistContentIdAndOrder>,

    // ISO 8601 형식의 String
    private val _createdAt: String = LocalDateTime.now().format(DateTimeFormatter.ISO_LOCAL_DATE_TIME)
) {
    val createdAt: LocalDateTime
        get() = LocalDateTime.parse(_createdAt, DateTimeFormatter.ISO_LOCAL_DATE_TIME)
}

data class AudioContentPlaylistContent @QueryProjection constructor(
    val id: Long,
    val title: String,
    val category: String,
    val coverUrl: String,
    val duration: String,
    val creatorNickname: String,
    val creatorProfileUrl: String
)
