package kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.like

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.QCreatorCommunity.creatorCommunity
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.like.QCreatorCommunityLike.creatorCommunityLike
import org.springframework.data.jpa.repository.JpaRepository

interface CreatorCommunityLikeRepository :
    JpaRepository<CreatorCommunityLike, Long>, CreatorCommunityLikeQueryRepository

interface CreatorCommunityLikeQueryRepository {
    fun findByPostIdAndMemberId(postId: Long, memberId: Long): CreatorCommunityLike?

    fun totalCountCommunityPostLikeByPostId(postId: Long): Int
}

class CreatorCommunityLikeQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : CreatorCommunityLikeQueryRepository {
    override fun findByPostIdAndMemberId(postId: Long, memberId: Long): CreatorCommunityLike? {
        return queryFactory
            .selectFrom(creatorCommunityLike)
            .innerJoin(creatorCommunityLike.creatorCommunity, creatorCommunity)
            .where(
                creatorCommunityLike.creatorCommunity.id.eq(postId)
                    .and(creatorCommunityLike.member.id.eq(memberId))
            )
            .fetchFirst()
    }

    override fun totalCountCommunityPostLikeByPostId(postId: Long): Int {
        return queryFactory
            .select(creatorCommunityLike.id)
            .from(creatorCommunityLike)
            .where(
                creatorCommunityLike.isActive.isTrue
                    .and(creatorCommunityLike.creatorCommunity.id.eq(postId))
            )
            .fetch()
            .size
    }
}
