package kr.co.vividnext.sodalive.live.room.detail

import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRole

data class GetRoomDetailResponse(
    val roomId: Long,
    val price: Int,
    val title: String,
    val notice: String,
    var isPaid: Boolean,
    val isAdult: Boolean,
    val isPrivateRoom: Boolean,
    val password: String?,
    val tags: List<String>,
    val channelName: String?,
    val beginDateTime: String,
    var numberOfParticipants: Int,
    val numberOfParticipantsTotal: Int
) {
    var manager: GetRoomDetailManager? = null
    var participatingUsers: List<GetRoomDetailUser> = listOf()
}

data class GetRoomDetailManager(
    val id: Long,
    val nickname: String,
    val introduce: String,
    val youtubeUrl: String?,
    val instagramUrl: String?,
    val websiteUrl: String?,
    val blogUrl: String?,
    val profileImageUrl: String,
    val isCreator: Boolean
) {
    constructor(member: Member, cloudFrontHost: String) : this(
        id = member.id!!,
        nickname = member.nickname,
        introduce = member.introduce,
        youtubeUrl = member.youtubeUrl,
        instagramUrl = member.instagramUrl,
        websiteUrl = member.websiteUrl,
        blogUrl = member.blogUrl,
        profileImageUrl = if (member.profileImage != null) {
            "$cloudFrontHost/${member.profileImage}"
        } else {
            "$cloudFrontHost/profile/default-profile.png"
        },
        isCreator = member.role == MemberRole.CREATOR
    )
}

data class GetRoomDetailUser(
    val id: Long,
    val nickname: String,
    val profileImageUrl: String
) {
    constructor(member: Member, cloudFrontHost: String) : this(
        member.id!!,
        member.nickname,
        profileImageUrl = if (member.profileImage != null) {
            "$cloudFrontHost/${member.profileImage}"
        } else {
            "$cloudFrontHost/profile/default-profile.png"
        }
    )
}
