package kr.co.vividnext.sodalive.live.tag

import kr.co.vividnext.sodalive.admin.member.tag.UpdateTagOrdersRequest
import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.DeleteMapping
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestPart
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@RequestMapping("/live/tag")
class LiveTagController(private val service: LiveTagService) {
    @PostMapping
    @PreAuthorize("hasRole('ADMIN')")
    fun enrollmentLiveTag(
        @RequestPart("image") image: MultipartFile,
        @RequestPart("request") requestString: String
    ) = ApiResponse.ok(service.enrollmentLiveTag(image, requestString), "등록되었습니다.")

    @DeleteMapping("/{id}")
    @PreAuthorize("hasRole('ADMIN')")
    fun deleteLiveTag(@PathVariable id: Long) = ApiResponse.ok(service.deleteTag(id), "삭제되었습니다.")

    @PutMapping("/{id}")
    @PreAuthorize("hasRole('ADMIN')")
    fun modifyLiveTag(
        @PathVariable id: Long,
        @RequestPart("image") image: MultipartFile?,
        @RequestPart("request") requestString: String
    ) = ApiResponse.ok(service.modifyTag(id, image, requestString), "수정되었습니다.")

    @PutMapping("/orders")
    @PreAuthorize("hasRole('ADMIN')")
    fun updateTagOrders(
        @RequestBody request: UpdateTagOrdersRequest
    ) = ApiResponse.ok(service.updateTagOrders(request.ids), "수정되었습니다.")

    @GetMapping
    fun getTags(
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) {
            throw SodaException("로그인 정보를 확인해주세요.")
        }

        ApiResponse.ok(service.getTags(member))
    }
}
