package kr.co.vividnext.sodalive.member.social.google

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.jwt.TokenProvider
import kr.co.vividnext.sodalive.member.MemberAdapter
import kr.co.vividnext.sodalive.member.MemberService
import kr.co.vividnext.sodalive.member.login.LoginResponse
import kr.co.vividnext.sodalive.member.social.SocialLoginResponse
import org.springframework.beans.factory.annotation.Value
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.stereotype.Service

@Service
class GoogleAuthService(
    private val googleService: GoogleService,
    private val memberService: MemberService,
    private val tokenProvider: TokenProvider,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) {
    fun authenticate(
        idToken: String,
        container: String,
        marketingPid: String?,
        pushToken: String?
    ): SocialLoginResponse {
        val googleUserInfo = googleService.getUserInfo(idToken)
            ?: throw SodaException("구글 로그인을 하지 못했습니다. 다시 시도해 주세요")
        val memberResolveResult = memberService.findOrRegister(googleUserInfo, container, marketingPid, pushToken)
        val member = memberResolveResult.member
        val principal = MemberAdapter(member)
        val authToken = GoogleAuthenticationToken(idToken, principal.authorities)
        authToken.setPrincipal(principal)
        SecurityContextHolder.getContext().authentication = authToken

        val jwt = tokenProvider.createToken(
            authentication = authToken,
            memberId = member.id!!
        )

        val loginResponse = LoginResponse(
            userId = member.id!!,
            token = jwt,
            nickname = member.nickname,
            email = member.email,
            profileImage = if (member.profileImage != null) {
                "$cloudFrontHost/${member.profileImage}"
            } else {
                "$cloudFrontHost/profile/default-profile.png"
            }
        )

        return SocialLoginResponse(
            memberId = member.id!!,
            marketingPid = marketingPid,
            loginResponse = loginResponse,
            isNew = memberResolveResult.isNew
        )
    }
}
