package kr.co.vividnext.sodalive.member.social.kakao

import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.common.SodaException
import org.springframework.http.HttpEntity
import org.springframework.http.HttpHeaders
import org.springframework.http.HttpMethod
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.stereotype.Service
import org.springframework.web.client.RestTemplate

@Service
class KakaoService(
    private val restTemplate: RestTemplate = RestTemplate(),
    private val objectMapper: ObjectMapper = ObjectMapper()
) {
    fun getUserInfo(accessToken: String): KakaoUserInfo? {
        val url = "https://kapi.kakao.com/v2/user/me"
        val headers = HttpHeaders().apply {
            set("Authorization", "Bearer $accessToken")
        }
        val entity = HttpEntity<Any>(headers)

        try {
            val response: ResponseEntity<String> = restTemplate.exchange(
                url,
                HttpMethod.GET,
                entity,
                String::class.java
            )

            if (response.statusCode == HttpStatus.OK) {
                // 응답 JSON을 KakaoUserInfo와 직접 매핑하려면 DTO의 구조와 JSON 구조가 일치해야 합니다.
                // 실제 응답 JSON은 중첩 구조를 가지므로, 필요한 필드만 추출하는 방법을 사용합니다.
                val jsonNode = objectMapper.readTree(response.body ?: return null)
                val id = jsonNode.get("id").asLong()
                val kakaoAccount = jsonNode.get("kakao_account")
                val email = kakaoAccount?.get("email")?.asText()
                    ?: throw SodaException("카카오 로그인을 하지 못했습니다. 다시 시도해 주세요")
                val properties = jsonNode.get("properties")
                val nickname = properties?.get("nickname")?.asText()

                return KakaoUserInfo(id, email, nickname)
            } else {
                println("카카오 사용자 정보 요청 실패: ${response.statusCode}")
            }
        } catch (ex: Exception) {
            ex.printStackTrace()
        }
        return null
    }
}
