package kr.co.vividnext.sodalive.point

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.point.QPointGrantLog.pointGrantLog
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface PointGrantLogRepository : JpaRepository<PointGrantLog, Long>, PointGrantLogQueryRepository

interface PointGrantLogQueryRepository {
    fun countByMemberIdAndPolicyIdAndStartDate(memberId: Long, policyId: Long, startDate: LocalDateTime): Int
}

class PointGrantLogQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : PointGrantLogQueryRepository {
    override fun countByMemberIdAndPolicyIdAndStartDate(memberId: Long, policyId: Long, startDate: LocalDateTime): Int {
        return queryFactory
            .select(pointGrantLog.id)
            .from(pointGrantLog)
            .where(
                pointGrantLog.memberId.eq(memberId),
                pointGrantLog.policyId.eq(policyId),
                pointGrantLog.createdAt.goe(startDate)
            )
            .fetch()
            .size
    }
}
