package kr.co.vividnext.sodalive.useraction

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.useraction.QUserActionLog.userActionLog
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface UserActionLogRepository : JpaRepository<UserActionLog, Long>, UserActionLogQueryRepository

interface UserActionLogQueryRepository {
    fun countByMemberIdAndActionTypeAndCreatedAtBetween(
        memberId: Long,
        actionType: ActionType,
        startDate: LocalDateTime,
        endDate: LocalDateTime
    ): Int
}

class UserActionLogQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : UserActionLogQueryRepository {
    override fun countByMemberIdAndActionTypeAndCreatedAtBetween(
        memberId: Long,
        actionType: ActionType,
        startDate: LocalDateTime,
        endDate: LocalDateTime
    ): Int {
        return queryFactory
            .select(userActionLog.id)
            .from(userActionLog)
            .where(
                userActionLog.memberId.eq(memberId)
                    .and(userActionLog.actionType.eq(actionType))
                    .and(userActionLog.createdAt.between(startDate, endDate))
            )
            .fetch()
            .size
    }
}
